import _ from 'lodash';
import Url from 'url-parse';
import queryString from 'query-string';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var StorageLimitType;

(function (StorageLimitType) {
  StorageLimitType["USER"] = "1";
  StorageLimitType["ORG"] = "2";
})(StorageLimitType || (StorageLimitType = {}));

var STORAGE_EXPIRED = 'expired';

var BlLocalStorage = /*#__PURE__*/function () {
  function BlLocalStorage() {
    var _this = this;

    _classCallCheck(this, BlLocalStorage);

    this.orgId = void 0;
    this.userId = void 0;

    this.setBaseInfo = function (userId, orgId) {
      _this.userId = userId;
      _this.orgId = orgId;
    };

    this.clearBaseInfo = function () {
      _this.userId = undefined;
      _this.orgId = undefined;
    };

    this.getLimitKey = function (limitType) {
      if (limitType === StorageLimitType.USER && _this.userId) {
        return "userId-".concat(_this.userId);
      }

      if (limitType === StorageLimitType.ORG && _this.orgId) {
        return "orgId-".concat(_this.orgId);
      }

      return '';
    };
  }

  _createClass(BlLocalStorage, [{
    key: "get",
    value: function get(key) {
      var localData = localStorage.getItem(key);

      if (localData) {
        var value = JSON.parse(localData);
        var now = new Date().getTime();
        var _value$meta = value.meta,
            storageTime = _value$meta.time,
            expiration = _value$meta.expiration,
            limitType = _value$meta.limitType;
        var isTimeIncompatible = expiration !== 0 && now - storageTime > expiration;

        if (limitType) {
          var strorageCurrentTime = _.get(value, "data[".concat(this.getLimitKey(limitType), "].time"));

          var isCurrentTimeIncompatible = expiration !== 0 && now - strorageCurrentTime > expiration;

          if (isCurrentTimeIncompatible) {
            this.remove(key);
            return STORAGE_EXPIRED;
          }

          return _.get(value, "data[".concat(this.getLimitKey(limitType), "].data"));
        } else if (isTimeIncompatible) {
          this.remove(key);
          return STORAGE_EXPIRED;
        }

        return value.data;
      }

      return undefined;
    }
  }, {
    key: "set",
    value: function set(key, data, option) {
      var _ref = option || {},
          _ref$expiration = _ref.expiration,
          expiration = _ref$expiration === void 0 ? 0 : _ref$expiration,
          limitType = _ref.limitType;

      var currentTime = new Date().getTime();
      var newValue = {
        meta: {
          expiration: expiration * 1000,
          time: currentTime,
          limitType: limitType
        },
        data: data
      };

      if (limitType === StorageLimitType.USER) {
        if (!this.userId) {
          console.error('设置用户限制类型的LocalStorage必须有用户信息');
          return;
        }

        newValue.data = _objectSpread2(_objectSpread2({}, this.getStorageData(key)), {}, _defineProperty({}, this.getLimitKey(limitType), {
          tiem: currentTime,
          data: data
        }));
      } else if (limitType === StorageLimitType.ORG) {
        if (!this.orgId) {
          console.error('设置工厂限制类型的LocalStorage必须有工厂信息');
          return;
        }

        newValue.data = _objectSpread2(_objectSpread2({}, this.getStorageData(key)), {}, _defineProperty({}, this.getLimitKey(limitType), {
          tiem: currentTime,
          data: data
        }));
      }

      localStorage.setItem(key, JSON.stringify(newValue));
    }
  }, {
    key: "remove",
    value: function remove(key) {
      var meta = this.getStorageMeta(key);

      if (meta) {
        var limitType = meta.limitType,
            expiration = meta.expiration;

        if (limitType) {
          this.set(key, undefined, {
            expiration: expiration,
            limitType: limitType
          });
        } else {
          localStorage.removeItem(key);
        }
      } else {
        localStorage.removeItem(key);
      }
    }
  }, {
    key: "clear",
    value: function clear() {
      localStorage.clear();
    }
  }, {
    key: "getStorageMeta",
    value: function getStorageMeta(key) {
      var storageJsonData = localStorage.getItem(key);

      if (storageJsonData) {
        var value = JSON.parse(storageJsonData);
        return value.meta;
      }

      return undefined;
    }
  }, {
    key: "getStorageData",
    value: function getStorageData(key) {
      var storageJsonData = localStorage.getItem(key);

      if (storageJsonData) {
        var value = JSON.parse(storageJsonData);
        return value.data;
      }

      return undefined;
    }
  }], [{
    key: "getInstance",
    value: function getInstance() {
      if (BlLocalStorage.instance === undefined) {
        BlLocalStorage.instance = new BlLocalStorage();
      }

      return BlLocalStorage.instance;
    }
  }]);

  return BlLocalStorage;
}();

BlLocalStorage.instance = void 0;

function getParams() {
  var _queryString$parseUrl = queryString.parseUrl(location.search, {
    arrayFormat: 'bracket-separator',
    arrayFormatSeparator: '|',
    parseNumbers: true
  }),
      formatParams = _queryString$parseUrl.query;

  return formatParams;
}

function setParams(params) {
  var _location = location,
      pathname = _location.pathname;
  var newUrl = queryString.stringifyUrl({
    url: pathname,
    query: params
  }, {
    arrayFormat: 'bracket-separator',
    arrayFormatSeparator: '|',
    skipNull: true
  });
  window.history.pushState(null, '', newUrl);
  return newUrl;
}

var getRelativePath = function getRelativePath(path) {
  if (!path) {
    return '/';
  }

  var url = new Url(decodeURIComponent(path));
  return "".concat(url.pathname).concat(url.query);
};

var getRouteParams = function getRouteParams(props) {
  return props.match.params;
};

var BlUrl = {};
BlUrl.getParams = getParams;
BlUrl.setParams = setParams;
BlUrl.getRelativePath = getRelativePath;
BlUrl.getRouteParams = getRouteParams;

export { BlLocalStorage, BlUrl, STORAGE_EXPIRED, StorageLimitType, getRelativePath };
